// tempus 2000.11g
#ifndef TELESCOPESYS_SYSTEM_CLASS
#define TELESCOPESYS_SYSTEM_CLASS
//
// (c) Copyright 1995-2001, MZA Associates Corporation
//

#include "tempus.h"


#include "Telescope.h"
#include "Slew.h"
#include "IncomingSplitter.h"
#include "SimpleFieldSensor.h"
#include "SquareWave.h"
#include "TransverseVelocity.h"

class TelescopeSys : public System
{
public:

   // Parameters
   float range;
   float tel_focus_range;
   float Dap;
   Vector<float> vplat;
   Vector<float> vtarg;
   float propdxy;
   double control_interval;
   float beacon_wavelength;

   // Subsystems
   Telescope telescope;

   Slew slew;

   IncomingSplitter is;

   SimpleFieldSensor sfs;

   SquareWave zoverc;

   TransverseVelocity transversevelocity;


   // Inputs
   Input< WaveTrain > outgoingIncident;
   Input< WaveTrain > incomingIncident;

   // Outputs
   Output< WaveTrain > outgoingTransmitted;
   Output< WaveTrain > incomingTransmitted;

private:      // Setting expressions for unsatisfied inputs/external outputs
   Output< double >  sfs_exposureInterval;
   Output< double >  sfs_exposureLength;
   Output< double >  sfs_sampleInterval;

public:
   TelescopeSys(SystemNode *parent, char *name,
                float _range,
                float _tel_focus_range,
                float _Dap,
                Vector<float> _vplat,
                Vector<float> _vtarg,
                float _propdxy,
                double _control_interval,
                float _beacon_wavelength) : 
      System(parent, name),
      range(_range),
      tel_focus_range(_tel_focus_range),
      Dap(_Dap),
      vplat(_vplat),
      vtarg(_vtarg),
      propdxy(_propdxy),
      control_interval(_control_interval),
      beacon_wavelength(_beacon_wavelength),
      telescope(this, "telescope",
         tel_focus_range,
         Dap/2.0,
         0.0),
      slew(this, "slew",
         (vtarg(0)-vplat(0))/range,
         (vtarg(1)-vplat(1))/range,
         -(vtarg(0)-vplat(0))/speedOfLight,
         -(vtarg(1)-vplat(1))/speedOfLight),
      is(this, "is"),
      sfs(this, "sfs",
         beacon_wavelength,
         Dap / propdxy,
         propdxy),
      zoverc(this, "zoverc",
         (range/speedOfLight),
         1.0,
         1.0e+06,
         1.0e+06),
      transversevelocity(this, "transversevelocity",
         vplat(0),
         vplat(1),
         0.0,
         0.0),
      outgoingIncident(this, "outgoingIncident"),
      incomingIncident(this, "incomingIncident"),
      outgoingTransmitted(this, "outgoingTransmitted"),
      incomingTransmitted(this, "incomingTransmitted"),
      sfs_exposureInterval(this, "sfs_exposureInterval"),
      sfs_exposureLength(this, "sfs_exposureLength"),
      sfs_sampleInterval(this, "sfs_sampleInterval")
   {
      slew.outgoingIncident <<= telescope.outgoingTransmitted;
      telescope.incomingIncident <<= slew.incomingTransmitted;
      is.incomingIncident <<= telescope.incomingTransmitted;
      telescope.outgoingIncident <<= is.outgoingTransmitted;
      sfs.incident <<= is.incomingTransmitted2;
      incomingTransmitted <<= is.incomingTransmitted;
      is.outgoingIncident <<= outgoingIncident;
      slew.incomingIncident <<= transversevelocity.incomingTransmitted;
      transversevelocity.outgoingIncident <<= slew.outgoingTransmitted;
      transversevelocity.incomingIncident <<= incomingIncident;
      outgoingTransmitted <<= transversevelocity.outgoingTransmitted;
      sfs.on <<= zoverc.boolOutputSignal;

      // System's Outputs and Subsystems' Inputs that
      // are not connected, but have setting expressions
      sfs.exposureInterval <<= sfs_exposureInterval;
      sfs.exposureLength <<= sfs_exposureLength;
      sfs.sampleInterval <<= sfs_sampleInterval;
   }

   // void respondToScheduledEvent (const Event & /* event */);
   // void respondToChangedInputs();
   // void respondToInputWarning (const InputBase* /* input */);
   void respondToOutputRequest (const OutputBase* output)
   {
      double t = now();
      if(*output == sfs_exposureInterval)
      {
         sfs_exposureInterval = (double)(control_interval);
      }
      else if(*output == sfs_exposureLength)
      {
         sfs_exposureLength = (double)(1.0e-6);
      }
      else if(*output == sfs_sampleInterval)
      {
         sfs_sampleInterval = (double)(-1.0);
      }
   }
};

#endif // TELESCOPESYS_SYSTEM_CLASS
